@@ -20,5 +20,5 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 # THE SOFTWARE.
 """import only accessible API wrappers"""
-from adafruit_bus_device.spi_device import SPIDevice
+from adafruit_bus_device.spi_device import SPIDevice  # type: ignore[import]
 from .cpy_spidev import SPIDevCtx  # for linux only
